import pytest
from fastapi import FastAPI
from fastapi.middleware.gzip import GZipMiddleware
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi.testclient import TestClient

from stopments import get_stoplight_elements_html
from stopments.embed import css_content, favicon_content, js_content


def fastapi_app():
    app = FastAPI(docs_url=None, redoc_url=None)

    @app.get("/")
    async def index():
        return {"message": "Hello, World!"}

    @app.get("/docs", include_in_schema=False)
    async def docs():
        html = get_stoplight_elements_html(
            openapi_url=app.openapi_url or "/openapi.json",
            title="API Documentation",
        )
        return HTMLResponse(content=html)

    return app


def fastapi_app_embed():
    app = FastAPI(docs_url=None, redoc_url=None)

    @app.get("/static/web-components.min.js", include_in_schema=False)
    async def web_components_js():
        return HTMLResponse(
            content=js_content,
            media_type="application/javascript; charset=utf-8",
        )

    @app.get("/static/styles.min.css", include_in_schema=False)
    async def styles_css():
        return HTMLResponse(
            content=css_content,
            media_type="text/css; charset=utf-8",
        )

    @app.get("/static/favicon.ico", include_in_schema=False)
    async def favicon_ico():
        return HTMLResponse(
            content=favicon_content,
            media_type="image/x-icon",
        )

    @app.get("/")
    async def index():
        return {"message": "Hello, World!"}

    @app.get("/docs", include_in_schema=False)
    async def docs():
        html = get_stoplight_elements_html(
            openapi_url=app.openapi_url or "/openapi.json",
            title="API Documentation",
            stoplight_elements_css_url="/static/styles.min.css",
            stoplight_elements_js_url="/static/web-components.min.js",
            stoplight_elements_favicon_url="/static/favicon.ico",
        )
        return HTMLResponse(content=html)

    return app


def fastapi_app_embed_gzip():
    app = fastapi_app_embed()
    app.add_middleware(GZipMiddleware, minimum_size=100)
    return app


def fastapi_app_staticfiles():
    app = FastAPI(docs_url=None, redoc_url=None)

    app.mount("/static", StaticFiles(packages=[("stopments", "static")]))

    @app.get("/")
    async def index():
        return {"message": "Hello, World!"}

    @app.get("/docs", include_in_schema=False)
    async def docs():
        html = get_stoplight_elements_html(
            openapi_url=app.openapi_url or "/openapi.json",
            title="API Documentation",
            stoplight_elements_css_url="/static/styles.min.css",
            stoplight_elements_js_url="/static/web-components.min.js",
            stoplight_elements_favicon_url="/static/favicon.ico",
        )
        return HTMLResponse(content=html)

    return app


def test_fastapi():
    app = fastapi_app()
    client = TestClient(app)
    response = client.get("/")
    assert response.status_code == 200
    assert response.json() == {"message": "Hello, World!"}

    response = client.get("/docs")
    assert response.status_code == 200
    assert "API Documentation" in response.text


@pytest.mark.parametrize(
    "app",
    [
        fastapi_app_embed(),
        fastapi_app_embed_gzip(),
        fastapi_app_staticfiles(),
    ],
)
def test_fastapi_embed(app: FastAPI):
    client = TestClient(app)
    response = client.get("/")
    assert response.status_code == 200
    assert response.json() == {"message": "Hello, World!"}

    response = client.get("/docs")
    assert response.status_code == 200
    assert "API Documentation" in response.text

    response = client.get("/static/web-components.min.js")
    assert response.status_code == 200

    response = client.get("/static/styles.min.css")
    assert response.status_code == 200

    response = client.get("/static/favicon.ico")
    assert response.status_code == 200



from typing import Any

import pytest

from stopments import (
    LayoutOptions,
    RouterOptions,
    TryItCredentialPolicyOptions,
    get_stoplight_elements_html,
)


def test_get_stoplight_elements_html():
    openapi_url = "https://api.apis.guru/v2/specs/github.com/1.1.4/openapi.json"
    title = "GitHub v3 REST API"
    html = get_stoplight_elements_html(
        openapi_url=openapi_url,
        title=title,
    )

    assert isinstance(html, str)
    assert openapi_url in html
    assert title in html


@pytest.mark.parametrize(
    ("kwargs", "include_str"),
    [
        (
            {
                "stoplight_elements_js_url": "https://unpkg.com/@stoplight/elements/web-components.min.js"
            },
            '"https://unpkg.com/@stoplight/elements/web-components.min.js"',
        ),
        (
            {
                "stoplight_elements_css_url": "https://unpkg.com/@stoplight/elements/styles.min.css"
            },
            '"https://unpkg.com/@stoplight/elements/styles.min.css"',
        ),
        (
            {
                "stoplight_elements_favicon_url": "https://docs.stoplight.io/favicons/favicon.ico"
            },
            '"https://docs.stoplight.io/favicons/favicon.ico"',
        ),
        (
            {
                "api_description_document": "openapi: 3.0.0\ninfo:\n  title: Test API\n  version: 1.0.0\npaths: {}"
            },
            "apiDescriptionDocument",
        ),
        ({"base_path": "/docs/api"}, "basePath="),
        ({"hide_internal": True}, "hideInternal="),
        ({"hide_try_it": True}, "hideTryIt="),
        ({"hide_export": True}, "hideExport="),
        ({"try_it_cors_proxy": "https://cors-proxy.example.com"}, "tryItCorsProxy="),
        (
            {"try_it_credential_policy": TryItCredentialPolicyOptions.OMIT},
            'tryItCredentialPolicy="omit"',
        ),
        ({"layout": LayoutOptions.RESPONSIVE}, 'layout="responsive"'),
        ({"logo": "https://example.com/logo.png"}, "logo="),
        ({"router": RouterOptions.HASH}, 'router="hash"'),
    ],
)
def test_includes(kwargs: dict[str, Any], include_str: str):
    openapi_url = "https://api.apis.guru/v2/specs/github.com/1.1.4/openapi.json"
    title = "GitHub v3 REST API"
    html = get_stoplight_elements_html(
        openapi_url=openapi_url,
        title=title,
        **kwargs,
    )
    assert include_str in html



from .__version__ import __version__
from .elements import (
    LayoutOptions,
    RouterOptions,
    TryItCredentialPolicyOptions,
    get_stoplight_elements_html,
)

__all__ = [
    "__version__",
    "LayoutOptions",
    "RouterOptions",
    "TryItCredentialPolicyOptions",
    "get_stoplight_elements_html",
]



from importlib.resources import files as _files

_MODULE = "stopments.static"

favicon_content = _files(_MODULE).joinpath("favicon.ico").read_bytes()
"""
Favicon content from https://docs.stoplight.io/favicons/favicon.ico

Content-Type: image/x-icon

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import favicon_content

app = FastAPI()

@app.get("/favicon.ico", include_in_schema=False)
async def favicon_ico():
    return HTMLResponse(content=favicon_content, media_type="image/x-icon")
```
"""


css_content = _files(_MODULE).joinpath("styles.min.css").read_bytes()
"""
CSS content from https://unpkg.com/@stoplight/elements/styles.min.css

Content-Type: text/css; charset=utf-8

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import css_content

app = FastAPI()

@app.get("/static/styles.min.css", include_in_schema=False)
async def styles_css():
    return HTMLResponse(content=css_content, media_type="text/css; charset=utf-8")
```
"""

js_content = _files(_MODULE).joinpath("web-components.min.js").read_bytes()
"""
JavaScript content from https://unpkg.com/@stoplight/elements/web-components.min.js

Content-Type: application/javascript; charset=utf-8

Example
-------

```python
from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from stopments.embed import js_content

app = FastAPI()

@app.get("/static/web-components.min.js", include_in_schema=False)
async def web_components_js():
    return HTMLResponse(
        content=js_content, media_type="application/javascript; charset=utf-8"
    )
```
"""

del _files, _MODULE

__all__ = ["favicon_content", "css_content", "js_content"]
